@@ -14,10 +14,14 @@
 #    under the License.
 import functools
 import json
+import logging
+import uuid
 
+import fixtures
 from keystoneauth1 import session
 from keystoneauth1 import token_endpoint
 import mock
+from oslo_utils import encodeutils
 import requests
 from requests_mock.contrib import fixture
 import six
@@ -148,6 +152,18 @@ class TestClient(testtools.TestCase):
         headers = self.mock.last_request.headers
         self.assertEqual(kwargs['language_header'], headers['Accept-Language'])
 
+    def test_request_id_header_passed(self):
+        global_id = encodeutils.safe_encode("req-%s" % uuid.uuid4())
+        kwargs = {'global_request_id': global_id}
+        http_client = http.HTTPClient(self.endpoint, **kwargs)
+
+        path = '/v2/images/my-image'
+        self.mock.get(self.endpoint + path)
+        http_client.get(path)
+
+        headers = self.mock.last_request.headers
+        self.assertEqual(global_id, headers['X-OpenStack-Request-ID'])
+
     def test_language_header_not_passed_no_language(self):
         kwargs = {}
         http_client = http.HTTPClient(self.endpoint, **kwargs)
@@ -205,6 +221,41 @@ class TestClient(testtools.TestCase):
         self.assertEqual(b"ni\xc3\xb1o", encoded[b"test"])
         self.assertNotIn("none-val", encoded)
 
+    @mock.patch('keystoneauth1.adapter.Adapter.request')
+    def test_http_duplicate_content_type_headers(self, mock_ksarq):
+        """Test proper handling of Content-Type headers.
+
+        encode_headers() must be called as late as possible before a
+        request is sent. If this principle is violated, and if any
+        changes are made to the headers between encode_headers() and the
+        actual request (for instance a call to
+        _set_common_request_kwargs()), and if you're trying to set a
+        Content-Type that is not equal to application/octet-stream (the
+        default), it is entirely possible that you'll end up with two
+        Content-Type headers defined (yours plus
+        application/octet-stream). The request will go out the door with
+        only one of them chosen seemingly at random.
+
+        This situation only occurs in python3. This test will never fail
+        in python2.
+        """
+        path = "/v2/images/my-image"
+        headers = {
+            "Content-Type": "application/openstack-images-v2.1-json-patch"
+        }
+        data = '[{"value": "qcow2", "path": "/disk_format", "op": "replace"}]'
+        self.mock.patch(self.endpoint + path)
+        sess_http_client = self._create_session_client()
+        sess_http_client.patch(path, headers=headers, data=data)
+        # Pull out the headers with which Adapter.request was invoked
+        ksarqh = mock_ksarq.call_args[1]['headers']
+        # Only one Content-Type header (of any text-type)
+        self.assertEqual(1, [encodeutils.safe_decode(key)
+                             for key in ksarqh.keys()].count(u'Content-Type'))
+        # And it's the one we set
+        self.assertEqual(b"application/openstack-images-v2.1-json-patch",
+                         ksarqh[b"Content-Type"])
+
     def test_raw_request(self):
         """Verify the path being used for HTTP requests reflects accurately."""
         headers = {"Content-Type": "text/plain"}
@@ -378,6 +429,22 @@ class TestClient(testtools.TestCase):
                         matchers.Not(matchers.MatchesRegex(token_regex)),
                         'token found in LOG.debug parameter')
 
+    def test_log_request_id_once(self):
+        logger = self.useFixture(fixtures.FakeLogger(level=logging.DEBUG))
+        data = "TEST"
+        path = '/v1/images/'
+        self.mock.get(self.endpoint + path, body=six.StringIO(data),
+                      headers={"Content-Type": "application/octet-stream",
+                               'x-openstack-request-id': "1234"})
+
+        resp, body = self.client.get(path)
+        self.assertIsInstance(body, types.GeneratorType)
+        self.assertEqual([data], list(body))
+        expected_log = ("GET call to image "
+                        "for http://example.com:9292/v1/images/ "
+                        "used request id 1234")
+        self.assertEqual(1, logger.output.count(expected_log))
+
     def test_expired_token_has_changed(self):
         # instantiate client with some token
         fake_token = b'fake-token'
